/********************************************************************
*  Licensed Materials - Property of IBM
*  
*  (c) Copyright IBM Corp.  2005, 2007 All Rights Reserved
*  
*  US Government Users Restricted Rights - Use, duplication or
*  disclosure restricted by GSA ADP Schedule Contract with
*  IBM Corp.
********************************************************************/

/*********************************************************************
 * FILE: %Z%%M%  %I%  %W% %G% %U%
 *
 * Description:
 *
 * Servlet that handles "forgot password".
 * Note that in order for a user to be able to use this feature, he
 * must have previously defined his challenge-response answers.
 *********************************************************************/

package examples.expi;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.itim.apps.ApplicationException;
import com.ibm.itim.apps.ITIMFailedLoginException;
import com.ibm.itim.apps.PlatformContext;
import com.ibm.itim.apps.identity.SelfPasswordManager;
import com.ibm.itim.apps.jaas.callback.PlatformCallbackHandler;

public class ForgotPasswordServlet extends HttpServlet {

	private static final long serialVersionUID = -7947473850048155469L;

	public static String LOGON = "logon.jsp";

	public static final String FORGOT_PWD = "forgotpwd.jsp";

	public static final String FORGOT_PWD_INFO = "forgotpwdinfo.jsp";

	public static final String USERID = "userid";

	public static final String ERROR_MESSAGE = "errorMessage";

	public static final String INFO_MESSAGE = "infoMessage";

	public static final String CHALLENGES = "challenges";

	public static final String PLATFORM_CALLBACK_HANDLER = "platformCallbackHandler";

	public static final String PLATFORM_CONTEXT = "platform";

	// The following get set in init()
	private static String userIDAttribute;

	private static boolean changeOnlyTIMPassword;

	private static boolean ssoEnabled;

	/**
	 * Read in the properties file.
	 *
	 * @see javax.servlet.GenericServlet#init()
	 */

	public void init() throws ServletException {
		ExpiUtil eu = new ExpiUtil();

		// Are we to only change the TIM Account password, or change all?
		String par = eu.getProperty(ExpiUtil.CHANGEONLYITIMPASSWORD).trim();
		changeOnlyTIMPassword = par.compareToIgnoreCase("true") == 0;
		log("init(): changeOnlyTIMPassword = " + changeOnlyTIMPassword);

		// Is SSO enabled?
		LOGON = eu.getPropertySSOCheck(ExpiUtil.LOGON_PAGE);
		ssoEnabled = eu.isSSOEnabled();

		/*
		 * If SSO is enabled, use the "username" attribute. Otherwise use
		 * whatever's set in the expiUtil.
		 */
		if (ssoEnabled)
			userIDAttribute = "username";
		else
			userIDAttribute = ExpiUtil.LOGON_ID;
		log("init(): userIDAttribute = " + userIDAttribute);

		log("ForgotPasswordServlet.init() complete");

	} // init

	/**
	 * Method destroy Right now this does nothing.
	 *
	 * @see javax.servlet.Servlet#destroy()
	 */

	public void destroy() {

	} // destroy

	/**
	 * Method doGet This just calls doPost().
	 *
	 * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest,
	 *      HttpServletResponse)
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {

		log("ForgotPasswordServlet.doGet() forwarding to doPost()");
		doPost(request, response);

	} // doGet

	/**
	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest,
	 *      HttpServletResponse)
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		request.setCharacterEncoding("UTF-8");

		log("ForgotPasswordServlet.doPost()");

		// Need a session -- at minimum to do error handling
		HttpSession session = request.getSession(true);

		// Find out how we got here and figure out what to do!
		// Note: no action specified means to get the challenges.
		String action = request.getParameter("action");
		log("ForgotPasswordServlet.doPost(): action = " + action);

		if (isNullOrEmpty(action)) {
			if (!session.isNew()) {
				log("[ForgotPasswordServlet.doPost:] invalidating previous session");
				session.invalidate();
				session = request.getSession();
			}
			doGetChallenges(request, response, session);
		} else if (action.equals("authenticate")) {
			doAuthenticate(request, response, session);
		} else {
			// Unknown action
		}

		return;

	} // doPost

	/**
	 * Method doAuthenticate. Attempt to authenticate the user via challenge
	 * response.
	 *
	 * @param request
	 * @param response
	 * @param session
	 * @throws ServletException
	 * @throws IOException
	 */
	private void doAuthenticate(HttpServletRequest request,
			HttpServletResponse response, HttpSession session)
			throws ServletException, IOException {

		// First check responses -- this will forward back to jsp
		Map cars = validateResponses(request, response, session);
		if ((cars == null) || cars.isEmpty())
			return;

		// Get the PlatformCallbackHandler and set the responses on it
		PlatformCallbackHandler pch = (PlatformCallbackHandler) session
				.getAttribute(PLATFORM_CALLBACK_HANDLER);
		if (pch == null) {
			log("doAuthenticate(): No PlatformCallbackHandler in the session");
			goToPage(LOGON, null, session, request, response);
			return;
		}
		pch.setResponses(cars);

		// Attempt to login; if it succeeds change the password!
		try {
			LoginContext lc = new LoginContext("ITIM", pch);
			lc.login();
			log("doAuthenticate(): LoginContext.login() succeeded");
			changePassword(request, response, session, lc.getSubject());
		} catch (LoginException e) {
			String eMsg ="";
			if ( e instanceof ITIMFailedLoginException) {
				eMsg = ((ITIMFailedLoginException)e).getMessageId();
			} else {
				eMsg = e.getMessage();
			}
			if (isNullOrEmpty(eMsg)) {
				goToPage(FORGOT_PWD,
						"An unknown error occurred.",
						session,
						request,
						response);
			} else if (eMsg.equals(ITIMFailedLoginException.INVALID_ANSWERS)) {
				//code to retrieve and check the number of attempts remaining
				int remainingAttempts = -1;
				try {
					remainingAttempts = pch.getRemainingAttempts();
				} catch(ApplicationException ex) {
					log(
						"doAuthenticate(): Caught ApplicationException: "
						+ ex.getMessage());
					goToPage(LOGON, ex.getMessage(), session, request, response);
				}
				if (remainingAttempts > -1) {
					ITIMFailedLoginException failedLoginException =
							new ITIMFailedLoginException(
							ITIMFailedLoginException.INVALID_ANSWERS_ATTEMPTS,
							new Object[]{Integer.toString(remainingAttempts)});

					log("doAuthenticate(): LoginContext.login() failed with error"
								+ failedLoginException.getMessage());
				goToPage(FORGOT_PWD,
						failedLoginException.getMessage(),
						session,
						request,
						response);
			} else {
				log("doAuthenticate(): LoginContext.login() failed with error"
								+ e.getMessage());
					goToPage(
						FORGOT_PWD,
						e.getMessage(),
						session,
						request,
						response);
				}

			} else {

				log("doAuthenticate(): LoginContext.login() failed with error"
							+ e.getMessage());
				goToPage(
					LOGON,
					e.getMessage(),
					session,
					request,
					response);
			}
		}

		return;
	}

	/**
	 * Method changePassword.
	 *
	 * @param request
	 * @param response
	 * @param session
	 * @param subject
	 * @throws ServletException
	 * @throws IOException
	 */
	private void changePassword(HttpServletRequest request,
			HttpServletResponse response, HttpSession session, Subject subject)
			throws ServletException, IOException {

		// First off, get a PasswordManager
		PlatformContext platform = (PlatformContext) session
				.getAttribute(PLATFORM_CONTEXT);
		if (platform == null) {
			log("changePassword(): No PlatformContext!");
			goToPage(
					LOGON,
					"Unable to reset password at this time.  Please try again later.",
					session, request, response);
			return;

		}

		SelfPasswordManager spm = new SelfPasswordManager(platform);
		String message = "You will be notified via email when the password change "
				+ "is complete. The email message will contain the new password.";

		try {
			spm.resetPassword(subject);
		} catch (Exception ex) {
			ex.printStackTrace();
			message = ex.toString();
		}
		goToInfoPage(FORGOT_PWD_INFO, message, session, request, response);
		return;

	} // changePassword

	/**
	 * Method validateResponses. Check to make sure that there was an answer for
	 * each response. Setup the forward if there were any problems.
	 *
	 * @param request
	 * @param response
	 * @param session
	 * @return Map
	 * @throws ServletException
	 * @throws IOException
	 */

	private Map<String, String> validateResponses(HttpServletRequest request,
			HttpServletResponse response, HttpSession session)
			throws ServletException, IOException {

		Map<String, String> cars = (Map<String, String>) session.getAttribute(CHALLENGES);
		Iterator it = cars.keySet().iterator();

		while (it.hasNext()) {
			String challenge = (String) it.next();
			log("e(): challenge: " + challenge);
			String responseToChallenge = (String) request
					.getParameter(challenge);
			log("validateResponses(): response: " + responseToChallenge);
			if (isNullOrEmpty(responseToChallenge)) {
				goToPage(
						FORGOT_PWD,
						"One or more responses was empty.  You must provide a response for each challenge.",
						session, request, response);
				return null;
			} else {
				cars.put(challenge, responseToChallenge);
			}
		}

		return cars;
	}

	/**
	 * Method doGetChallenges. Get the challenges for the userid that was
	 * specified in the request, set them in the session, and forward to
	 * forgotpwd.jsp.
	 *
	 * @param request
	 * @param response
	 * @param session
	 * @throws ServletException
	 * @throws IOException
	 */

	private void doGetChallenges(HttpServletRequest request,
			HttpServletResponse response, HttpSession session)
			throws ServletException, IOException {

		log("doGetChallenges() begin");

		// Get the userID from the request; if there wasn't one
		// this will handle setting up the forward back to the logon page.
		String userID = getUserID(request, response, session);
		if (userID == null)
			return;

		Map challenges = null;
		try {
			// First off, setup PlatformCallbackHandler
			PlatformCallbackHandler pch = new PlatformCallbackHandler(userID,
					Locale.US);
			ExpiUtil eu = new ExpiUtil();
			PlatformContext platform = eu.createPlatformContext();
			pch.setPlatformContext(platform);
			challenges = pch.getUserChallenges();

			session.setAttribute(CHALLENGES, challenges);
			session.setAttribute(PLATFORM_CALLBACK_HANDLER, pch);
			session.setAttribute(PLATFORM_CONTEXT, platform);
			session.setAttribute(ExpiUtil.LOGON_ID, userID);

			/*
			 * See if there are challenges.
			 *
			 * NOTE: If challenges come back empty but there are no errors, we
			 * can't tell if it's because C/R is disabled or if the user just
			 * doesn't have any challenges defined. The API that we could use to
			 * find out requires a Subject.
			 */
			if ((challenges == null) || challenges.isEmpty()) {
				goToPage(
						LOGON,
						"Either you do not have any challenges configured, or challenge-resonse is disabled.",
						session, request, response);
			} else {
				goToPage(FORGOT_PWD, null, session, request, response);
			}
		} catch (ApplicationException e) {
			log("getChallenges(): Caught ApplicationException: "
					+ e.getMessage());
			goToPage(LOGON, e.getMessage(), session, request, response);
		} catch (RemoteException e) {
			log("getChallenges(): Caught RemoteException: " + e.getMessage());
			goToPage(LOGON, e.getMessage(), session, request, response);
		} catch (FailedLoginException e) {
			log("getChallenges(): Caught FailedLoginException: " + e.getMessage());
			goToPage(LOGON, e.getMessage(), session, request, response);
		}

		log("doGetChallenges() complete");

	} // getChallenges

	/**
	 * Method getUserID. Get the userID from the request based on the setting of
	 * userIDAttribute. If there isn't one, setup the forward back to the logon
	 * page.
	 *
	 * Note: the logon page checks to make sure that the userID has a value. But
	 * the user might have tried to get here directly.
	 *
	 * @param request
	 * @param response
	 * @param session
	 * @return String
	 * @throws ServletException
	 * @throws IOException
	 */

	private String getUserID(HttpServletRequest request,
			HttpServletResponse response, HttpSession session)
			throws ServletException, IOException {

		String userID = (String) request.getParameter(userIDAttribute);

		if (userID == null) {
			goToPage(
					LOGON,
					"Enter your User ID and then click on the \"Forgot Your Password\" link",
					session, request, response);
		}

		log("ForgotPasswordServlet.getUserID(): userID = " + userID);

		return userID;

	} // getUserID

	/**
	 * Method goToPage. Set the message in the request and forward it on to the
	 * specified page.
	 *
	 * @param page
	 * @param errorMessage
	 * @param session
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */

	private void goToPage(String page, String errorMessage,
			HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		session.setAttribute(ERROR_MESSAGE, errorMessage);

		RequestDispatcher dispatcher = getServletContext()
				.getRequestDispatcher("/" + page);

		dispatcher.forward(request, response);

	} // goToPage

	/**
	 * Method goToInfoPage. Set the message in the request and forward it on to
	 * the specified page.
	 *
	 * @param page
	 * @param infoMessage
	 * @param session
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */

	private void goToInfoPage(String page, String infoMessage,
			HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		session.setAttribute(INFO_MESSAGE, infoMessage);

		if (ssoEnabled) {
			request.setAttribute("ERROR", infoMessage);
		}

		RequestDispatcher dispatcher = getServletContext()
				.getRequestDispatcher("/" + page);

		dispatcher.forward(request, response);

	} // goToPage

	/**
	 * Method isNullOrEmpty. Returns true if the given object is null or equals
	 * "".
	 *
	 * @param o
	 * @return boolean
	 */
	private boolean isNullOrEmpty(Object o) {

		if ((o == null) || (o.equals("")))
			return true;
		else
			return false;

	} // isNullOrEmpty

	// /**
	// * Method validateSessionData. Validates the session data. If anything is
	// * amiss, forward back to the logon page.
	// *
	// * @param session
	// * @param request
	// * @param response
	// * @param subject
	// * @param platform
	// * @return boolean
	// * @throws ServletException
	// * @throws IOException
	// */
	// private boolean validateSessionData(HttpSession session,
	// HttpServletRequest request, HttpServletResponse response,
	// Subject subject, PlatformContext platform) throws ServletException,
	// IOException {
	//
	// log("ForgotPasswordServlet.validateSessionData(): start");
	//
	// boolean isValid = true;
	//
	// if ((subject == null) || (platform == null)) {
	// goToPage(LOGON, "Your session is invalid, please login again.",
	// session, request, response);
	// isValid = false;
	// }
	//
	// return isValid;
	//
	//	} // validateSessionData

	// /**
	// * Method getAccountsToChange.
	// *
	// * Consults the changeOnlyTIMPassword property and retrieves the
	// appropriate
	// * accounts.
	// *
	// * @param session
	// * @param request
	// * @param response
	// * @param pm
	// * @param pmo
	// * @return Collection
	// * @throws ServletException
	// * @throws IOException
	// */
	// private Collection getAccountsToChange(HttpSession session,
	// HttpServletRequest request, HttpServletResponse response,
	// PasswordManager pm, PersonMO pmo) throws ServletException,
	// IOException {
	//
	// log("getAccountsToChange(): start");
	//
	// Collection accounts = null;
	//
	// try {
	// accounts = pm.getPasswordAccounts(pmo);
	//
	// if (changeOnlyTIMPassword == true) {
	// accounts = getTIMAccount(session, request, response, accounts);
	// }
	//
	// // Setup the forward if needed
	// if ((accounts == null) || accounts.isEmpty()) {
	// log("getAccountsToChange(): Accounts is null or no Accounts found");
	// goToPage(
	// LOGON,
	// "You have answered the challenge questions correctly, however, your
	// password cannot be changed because your accounts cannot be retrieved.
	// Please try again later.",
	// session, request, response);
	// }
	//
	// } catch (ApplicationException e) {
	//
	// /*
	// * Thrown if unable to obtain the accounts. This may possibly be
	// * caused by the subject being removed by another client previous to
	// * this call. Note that since the user is logged in, they should
	// * have at least one account!
	// */
	// log("getAccountsToChange(): Caught ApplicationException: "
	// + e.getMessage());
	// goToPage(
	// LOGON,
	// "You have answered the challenge questions correctly, however, your
	// password cannot be changed because your accounts cannot be retrieved.
	// Please try again later.",
	// session, request, response);
	//
	// } catch (RemoteException e) {
	//
	// log("ChangePasswordServlet.getAccountsToChange(): Caught RemoteException:
	// "
	// + e.getMessage());
	//
	// // Thrown if unable to communicate with platform.
	// goToPage(
	// FORGOT_PWD,
	// "Unable to retrieve your accounts. The server may be busy or there may be
	// a more serious problem. Please try again. If the problem persists,
	// contact your system administrator.",
	// session, request, response);
	//		}
	//
	//		return accounts;
	//
	//	} // getAccountsToChange

	// /**
	// * Method getTIMAccount. Returns a Collection that contains the TIM
	// Account
	// * of the user who is trying to reset his password. Note that the user
	// could
	// * have more than one TIM Account, that's why we have to do the check.
	// *
	// * @param session
	// * @param request
	// * @param response
	// * @param accounts
	// * @return Collection
	// * @throws ServletException
	// * @throws IOException
	// */
	// private Collection getTIMAccount(HttpSession session,
	// HttpServletRequest request, HttpServletResponse response,
	// Collection accounts) throws ServletException, IOException {
	//
	// String userID = (String) session.getAttribute(ExpiUtil.LOGON_ID);
	// log("getTIMAccount(): user ID = " + userID);
	// if (userID == null) {
	// log("getTIMAccount(): could not get userID from session");
	// return null;
	// }
	//
	// Collection onlyTimAccount = new Vector();
	// Iterator it = accounts.iterator();
	//
	// try {
	// while (it.hasNext()) {
	// AccountMO amo = (AccountMO) it.next();
	//
	// /*
	// * Note: SERVICE_ENROLE_PROFILE_NAME = "ITIMService" This is the
	// * service name of the account. If you were searching for an
	// * Solaris account, you would put that service name here
	// * instead.
	// */
	// log("getTIMAccount(): Profile name "
	// + amo.getService().getData().getServiceProfileName());
	// if (amo.getService().getData().getServiceProfileName().equals(
	// ExpiUtil.SERVICE_ENROLE_PROFILE_NAME)) {
	// log("getTIMAccount(): AccountMO user ID: "
	// + amo.getData().getUserId());
	// String aUserID = amo.getData().getUserId();
	// if (aUserID.equalsIgnoreCase(userID)) {
	// onlyTimAccount.add(amo);
	// return onlyTimAccount;
	// }
	// }
	// } // while
	// } catch (Exception e) {
	// accounts = null;
	// }
	//
	// if (onlyTimAccount == null) {
	// log("getTIMAccount(): Could not find TIM Account for user.");
	// }
	//
	//		return onlyTimAccount;
	//
	//	} // getTIMAccount

} // ForgotPasswordServlet
